
import math
import VISI

# ===========================================================
# CAM system functions library 
# ===========================================================
# -----------------------------------------------------------------------
# This function determinates if a feature is drilled (conic tip at the end)
# by returning 1 or else by returning 0. 
# Fea_SubType:
#  0 = Blind
#  1 = Through
#  2 = Double blind (Not Used)
#  3 = Unknown (Not Used)
#  4 = Conic tip
#  5 = Radiused
#
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $Fea_MachinableDepth+(3*IsDril)

def IsDrld(Fea_SubType):
	if(Fea_SubType==4):
		return 1
	else:
		return 0
# -----------------------------------------------------------------------
# This function determinates if a feature is blind by returning 1 
# or else by returning 0. 
# Fea_SubType:
#  0 = Blind
#  1 = Through
#  2 = Double blind (Not Used)
#  3 = Unknown (Not Used)
#  4 = Conic tip
#  5 = Radiused
#
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $Fea_MachinableDepth+(3*IsBlnd)

def IsBlnd(Fea_SubType):
	if(Fea_SubType==0):
		return 1
	else:
		return 0
# -----------------------------------------------------------------------
# This function determinates if a feature is through by returning 1 
# or else by returning 0. 
# Fea_SubType:
#  0 = Blind
#  1 = Through
#  2 = Double blind (Not Used)
#  3 = Unknown (Not Used)
#  4 = Conic tip
#  5 = Radiused
#
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $Fea_MachinableDepth+(3*IsThru)

def IsThru(Fea_SubType):
	if(Fea_SubType==1):
		return 1
	else:
		return 0
# -----------------------------------------------------------------------
# This function calculates the inverse tangent of a given angle.
# This information can be used to determinate the correct depth of 
# drilling operation based on a given diameter and a tool tip angle.
# The formula can also be used for milling tool such as the chamfer (19)
# in this case the Tool_TipAngle variable si already divided.
# Based on the 2nd. theorem on rectangular triangle:
#  C = B * (1/tan(angle))
#
#  C = Operation depth
#  B = Given diameter/2
#  (angle) = Tool tip angle/2  (if exists)
#
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $(Fea_Diameter/2)*(1/Tangle)

def Tangle(Tool_TipAngle,Tool_Type):
	if(Tool_TipAngle>0.0 and Tool_TipAngle<180.0):
		if(Tool_Type==1):
			return math.tan(math.radians(Tool_TipAngle/2))
		else:
			return math.tan(math.radians(Tool_TipAngle))
	else:
		return 1.0
# -----------------------------------------------------------------------
# This function calculates the tool tip height.
# Based on the 2nd. theorem on rectangular triangle:
#  C = B * (1/tan(angle))
#
#  C = Tool tip height
#  B = Tool diameter/2
#  (angle) = Tool tip angle/2  (if exists)
#
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $Fea_Depth+TipHeight
# (modify from Paolo Favro 22/10/2013) the TipHeigh is calculated only for a drill tools excluded the Drill Flat tool  (bug-zilla 25649).
# Tool_Type =1 drill
# Tool_Subtype = 103 flat drill

def TipHeight(Tool_TipAngle,Tool_Diameter,Tool_Type,Tool_SubType):
	if(Tool_TipAngle>0.0 and Tool_TipAngle<180.0)and(Tool_Type==1)and(Tool_SubType!=103):
		return ((Tool_Diameter/2)*(1/(math.tan(math.radians(Tool_TipAngle/2)))))
	else:
		return 0.0
# -----------------------------------------------------------------------
# This function calculates the absolute difference between the 
# step start position and the multistep start position. This number
# can then be added as compensation to the operation depth
# when the operation itself starts from the multistep start postion
#vs. the step start postion.
# Example:
#  [COMPASS]
#  VCYC_StartingZ = 0 ;
#  VCYC_Depth = 0 ; $Fea_MachinableDepth+(3*IsThru)+ZStep

def Step(Fea_TotalStartPositionZ,Fea_StartPositionZ):
	return math.fabs(Fea_TotalStartPositionZ-Fea_StartPositionZ)
# -----------------------------------------------------------------------
# This function determinates if a the current feature has a 
# thread diameter by returning 1 or else by returning 0. This 
# function can be used to discriminate from which diameters the mark
# diameter should be applied.
# Example:
#  [COMPASS]
#  VCYC_Depth = 0 ; $(((Fea_Diameter*IsThread)+(Fea_ThreadDiam*IsThread))/2) *(1/Tangle)

def IsThread(Fea_ThreadDiam):
	if (Fea_ThreadDiam>0.0):
		return 1
	else:
		return 0
# -----------------------------------------------------------------------
# This function determinates if a the current stepover inherited from the 
# cutting condition page has a valid number or not.
# Example:
#  [COMPASS]
#  V2AX_StepOver = 0 ; $((Cc_StepOver*IsCcOver)+((Tool_Diameter/2*(1-IsCcOver)))

def IsCcOver(Cc_StepOver):
	if (Cc_StepOver<=0.0):
		return 0
	else:
		return 1
# -----------------------------------------------------------------------
# This function determinates if a the current stepdown inherited from the 
# cutting condition page has a valid number or not.
# Example:
#  [COMPASS]
#  V2AX_StepDown = 0 ; $((Cc_StepDown*IsCcDown)+((Tool_Diameter/4)*(1-IsCcDown)))

def IsCcDown(Cc_StepDown):
	if (Cc_StepDown<=0.0):
		return 0
	else:
		return 1
# -----------------------------------------------------------------------
# This function determinates if a the current value for the Fea_BottleNeck
# is bigger than (0) zero.
# Example:
#  [COMPASS]
#  # TOOL_MAX_DIAM       = 0 ; $Fea_BottleNeck+(Fea_Width*(1-IsBotNeck))

def IsBotNeck(Fea_BottleNeck):
	if(Fea_BottleNeck<=0.0):
		return 0
	else:
		return 1

# -----------------------------------------------------------------------
# This function assume the largerst value between the 2 diameters
# a hole feature potentailly has. (Fea_Diameter vs. Fea_ThradDiam)determinates if a the current value for the Fea_BottleNeck

def Diam(Fea_Diameter,Fea_ThreadDiam):
	return max(Fea_Diameter,Fea_ThreadDiam)
# -----------------------------------------------------------------------
# This function determinates the depth of the drilling operation. 
# The control is centralised for tool & operation combination

def DefaultDepth (Fea_MachinableDepth,Fea_Depth,Fea_ThreadDepth,Fea_Pitch,Fea_SubType,Fea_TotalStartPositionZ,Fea_StartPositionZ,Fea_Diameter,Fea_ThreadDiam,Tool_Type,Tool_TipAngle,Tool_Diameter,Tool_SubType,VCYC_MarkDiam,USR_DEPTH,USR_DEPTHBLND,USR_NPITCH,USR_CHFR,USR_CENTFIXDEPTH,Fea_TipAngle,USR_DEPTHR):
			
	# Variable preparation
	
	Step = math.fabs(Fea_TotalStartPositionZ-Fea_StartPositionZ)
	
	if (Fea_TipAngle>0.0 and Fea_TipAngle<180.0):
	
		TipHeightFea = ((Fea_Diameter/2)*(1/(math.tan(math.radians(Fea_TipAngle/2)))))
	
	else:
		
		TipHeightFea = 0.0
		
	if (Tool_TipAngle>0.0 and Tool_TipAngle<180.0)and(Tool_Type==1):
		
		TipHeight = ((Tool_Diameter/2)*(1/(math.tan(math.radians(Tool_TipAngle/2)))))
		
	else:
		TipHeight = 0.0
	
	# Drilling tool 
	# 100 = Drill
	
	if (Tool_SubType == 100):
					
		# Blind management
		if (Fea_SubType == 0):
			return (Fea_MachinableDepth+USR_DEPTHBLND+Step)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_MachinableDepth+USR_DEPTH+Step+TipHeight)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_Depth+Step+min(TipHeight,TipHeightFea))
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_MachinableDepth+Step)
			
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Tap tool
	# 200 = Tap
	# 201 = Thread mill
	
	if (Tool_SubType == 200 or Tool_SubType == 201):

		# Blind management
		if (Fea_SubType == 0):
			return (Fea_ThreadDepth+Step)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_ThreadDepth+(USR_NPITCH*Fea_Pitch)+Step)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_ThreadDepth+Step)
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_ThreadDepth+Step)	
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Boring bar
	# 152 = Boring bar
	
	if (Tool_SubType == 152):

		# Blind management
		if (Fea_SubType == 0):
			return (Fea_Depth+Step)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_Depth+Step+USR_DEPTH)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_Depth+Step)
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_Depth+Step)	

	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Reamer
	# 150 = Reamer
	# 151 = Adjustable reamer

	if (Tool_SubType == 150 or Tool_SubType == 151):

		# Blind management
		if (Fea_SubType == 0):
			return (Fea_Depth+Step-Fea_Diameter*USR_DEPTHR)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_Depth+Step+USR_DEPTH)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_Depth+Step-Fea_Diameter*USR_DEPTHR)
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_Depth+Step)
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Center drill
	# 50 = Centre drill
	
	if (Tool_SubType == 50):
		if(Tool_TipAngle>0.0 and Tool_TipAngle<180.0):
			
			RefDiam = max(Fea_Diameter,Fea_ThreadDiam)
		
			return (((RefDiam+USR_CHFR)/2)*(1/(math.tan(math.radians(Tool_TipAngle/2)))))
		else:
			return 0.0
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Center drill
	# 51 = Centre & chamfering drill
	
	if (Tool_SubType == 51):
		
		return VCYC_MarkDiam 
		
	#	if(VCYC_MarkDiam>=Tool_Diameter):
	#		RefDiam = Tool_Diameter
	#	else:
	#		RefDiam = VCYC_MarkDiam
	#		
	#	return VCYC_MarkDiam 
		#((RefDiam/2)*(1/(math.tan(math.radians(Tool_TipAngle/2)))))
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Spade drill
	# 102 = Countersink (Spade drill)
		
	if (Tool_SubType == 102):
					
		# Blind management
		if (Fea_SubType == 0):
			return (Fea_MachinableDepth+USR_DEPTHBLND+Step)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_MachinableDepth+USR_DEPTH+Step+TipHeight)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_Depth+Step+min(TipHeight,TipHeightFea))
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_MachinableDepth+Step)
			
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Generic tool 
	# 101 = U drill
	# 103 = Flat bottom drill
	# 104 = Drill 2 diameters
	
	if (Tool_SubType == 101 or Tool_SubType == 103 or Tool_SubType == 104):

		# Blind management
		if (Fea_SubType == 0):
			return (Fea_Depth+Step)
		
		# Through management
		if (Fea_SubType == 1):
			return (Fea_Depth+Step+USR_DEPTH)
	
		# Conic tip (drilled) management
		if (Fea_SubType == 4):
			return (Fea_Depth+Step)
			
		# Radiused management
		if (Fea_SubType == 5):
			return (Fea_Depth+Step)	
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	else:
		return (Fea_Depth+Step)
# -----------------------------------------------------------------------
# This function determinates the way the "tool tip" query is controlled 
# if the feature has a tip angle then the same value is used for the query
# otherwise the query will search for a tool with any tip angle 

def DefaultAngleQry(Fea_TipAngle,Fea_SubType,Tool_SubType,Fea_VolumeType):
	
	#Managment for chamfer milling tool used on conical features
	if(Fea_VolumeType==7 and Tool_SubType == 19):
		return Fea_TipAngle/2
			
	#Managment for drilling tool on regular hole features
	if(Fea_VolumeType==1 and Fea_SubType == 4):
			return Fea_TipAngle
	
	return -1
		
# -----------------------------------------------------------------------
